#ifndef TD3DLightH
#define TD3DLightH

// ==========================================================================
// File:         TD3DLight.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DLight Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TD3DLight properties.
#include "TD3DColorValue.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DLight
// Description: The TD3DLight component wraps the DirectX D3DLIGHT7 
//              structure, which defines the properties of a light source. 
//              
//              This component is used in the TDx_3DDevice::GetLight() and 
//              TDx_3DDevice::SetLight() methods when defining and retrieving 
//              lighting properties. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DLight : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DLight : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Ambient
    // Description: The Ambient property contains a TD3DColorValue component 
    //              defining the ambient color emitted by the light. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* Ambient = { read=FGetAmbient, 
    write=FSetAmbient, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Diffuse
    // Description: The Diffuse property contains a TD3DColorValue component 
    //              defining the diffuse color emitted by the light. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* Diffuse = { read=FGetDiffuse, 
    write=FSetDiffuse, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Specular
    // Description: The Specular property contains a TD3DColorValue component 
    //              defining the specular color emitted by the light. 
    // ----------------------------------------------------------------------
    __property TD3DColorValue* Specular = { read=FGetSpecular, 
    write=FSetSpecular, default=NULL }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DLight component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Attenuation0
    // Description: The Attenuation0 property contains a D3DVALUE defining 
    //              the light's constant attentuation factor, which in turn 
    //              helps define how a light's intensity varies over 
    //              distance. 
    //              
    //              Attenuation does not affect directional lights and can 
    //              range from 0.0 to infinity. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Attenuation0 = { read=FGetAttenuation0, 
    write=FSetAttenuation0, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Attenuation1
    // Description: The Attenuation1 property contains a D3DVALUE defining 
    //              the light's linear attentuation factor, which in turn 
    //              helps define how a light's intensity varies over 
    //              distance. 
    //              
    //              Attenuation does not affect directional lights and can 
    //              range from 0.0 to infinity. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Attenuation1 = { read=FGetAttenuation1, 
    write=FSetAttenuation1, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Attenuation2
    // Description: The Attenuation2 property contains a D3DVALUE defining 
    //              the light's quadratic attentuation factor, which in turn 
    //              helps define how a light's intensity varies over 
    //              distance. 
    //              
    //              Attenuation does not affect directional lights and can 
    //              range from 0.0 to infinity. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Attenuation2 = { read=FGetAttenuation2, 
    write=FSetAttenuation2, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Direction
    // Description: The Direction property contains a TD3DVector component 
    //              indicating the direction the light is pointing in world 
    //              space. 
    //              
    //              This propety only has meaning for the directional and 
    //              spotlight types of light source. 
    //              This vector need not be normalized but should be of non 
    //              zero length. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR Direction = { read=FGetDirection, 
    write=FSetDirection, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DLight method or fget/fset. 
    //              eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Falloff
    // Description: The Falloff property contains a D3DVALUE representing the 
    //              decrease in illumination from a spotlight's umbra, 
    //              defined by the Theta property, and the outer edge of the 
    //              penumbra, defined by the Phi property. 
    //              
    //              A value of 1.0 will result in a linear falloff from the 
    //              umbra to the penumbra. 
    //              Values greater than 1.0 will cause the light to fall off 
    //              quickly at first and then fade slowly to the penumbra 
    //              while a value less than 1.0 will create the opposite 
    //              effect. 
    //              The effect of non 1.0 values is subtle and affects 
    //              performance a little, thus a 1.0 value is generally 
    //              reccomended. 
    //              
    // ----------------------------------------------------------------------
    __property D3DVALUE Falloff = { read=FGetFalloff, write=FSetFalloff, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Phi
    // Description: The Phi property contains a D3DVALUE defining the angle, 
    //              in radians, representing the outer edge of the 
    //              spotlight's penumbra. 
    //              
    //              Points outside the penumbra are not lit by the spotlight. 
    //              Points inside the penumbra and outside the umbra are 
    //              illuminated with an intensity relative to their position 
    //              and the Falloff value. 
    //              The value can range from 0 to pi and must equal or exceed 
    //              the value of the Theta property. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Phi = { read=FGetPhi, write=FSetPhi, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Position
    // Description: Contains a TD3DVector indicating the position of the 
    //              light source in world space. 
    //              This property has no meaning for a directional type light 
    //              source. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR Position = { read=FGetPosition, write=FSetPosition, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Range
    // Description: The Range property contains a D3DVALUE defining the 
    //              maximum distance a light source may have an effect. 
    //              
    //              This property does not affect directional light sources. 
    //              The maximum value is D3DLIGHT_RANGE_MAX, which is the 
    //              square root of FLT_MAX. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Range = { read=FGetRange, write=FSetRange, nodefault 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DLIGHT7 structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Theta
    // Description: The Theta property contains a D3DVALUE defining the 
    //              angle, in radians, representing the outer edge of the 
    //              spotlight's umbra. 
    //              
    //              Points inside the umbra are fully illuminated by the 
    //              spotlight. 
    //              Points outside the umbra and inside the penumbra are 
    //              illuminated with an intensity relative to their position 
    //              and the Falloff value. 
    //              The value can range from 0 to the value of the Phi 
    //              property. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Theta = { read=FGetTheta, write=FSetTheta, nodefault 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Type
    // Description: The Type property defines the type of light source being 
    //              described. 
    //              
    //              The possible values are: 
    //              D3DLIGHT_POINT indicating a point source. 
    //              D3DLIGHT_SPOT indicating a spotlight source. 
    //              D3DLIGHT_DIRECTIONAL indicating a directional source. 
    // ----------------------------------------------------------------------
    __property D3DLIGHTTYPE Type = { read=FGetType, write=FSetType, nodefault 
    }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DLight's internal D3DLIGHT7 structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DLight(TComponent* Owner); 
    virtual __fastcall ~TD3DLight(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DLIGHT7* Internal_D3DLIGHT7_Ptr = { 
    read=FGetInternal_D3DLIGHT7_Ptr, nodefault }; 

    void __fastcall Internal_D3DLIGHT7_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    TD3DColorValue* __fastcall FGetAmbient(); 
    void __fastcall FSetAmbient( TD3DColorValue* pAmbient ); 
    D3DVALUE __fastcall FGetAttenuation0(); 
    void __fastcall FSetAttenuation0( D3DVALUE pAttenuation0 ); 
    D3DVALUE __fastcall FGetAttenuation1(); 
    void __fastcall FSetAttenuation1( D3DVALUE pAttenuation1 ); 
    D3DVALUE __fastcall FGetAttenuation2(); 
    void __fastcall FSetAttenuation2( D3DVALUE pAttenuation2 ); 
    TD3DColorValue* __fastcall FGetDiffuse(); 
    void __fastcall FSetDiffuse( TD3DColorValue* pDiffuse ); 
    D3DVECTOR __fastcall FGetDirection(); 
    void __fastcall FSetDirection( D3DVECTOR pDirection ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    D3DVALUE __fastcall FGetFalloff(); 
    void __fastcall FSetFalloff( D3DVALUE pFalloff ); 
    D3DVALUE __fastcall FGetPhi(); 
    void __fastcall FSetPhi( D3DVALUE pPhi ); 
    D3DVECTOR __fastcall FGetPosition(); 
    void __fastcall FSetPosition( D3DVECTOR pPosition ); 
    D3DVALUE __fastcall FGetRange(); 
    void __fastcall FSetRange( D3DVALUE pRange ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    TD3DColorValue* __fastcall FGetSpecular(); 
    void __fastcall FSetSpecular( TD3DColorValue* pSpecular ); 
    D3DVALUE __fastcall FGetTheta(); 
    void __fastcall FSetTheta( D3DVALUE pTheta ); 
    D3DLIGHTTYPE __fastcall FGetType(); 
    void __fastcall FSetType( D3DLIGHTTYPE pType ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DLIGHT7* __fastcall FGetInternal_D3DLIGHT7_Ptr(); 
    void __fastcall FSetInternal_D3DLIGHT7( D3DLIGHT7* pD3DLIGHT7 ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    TD3DColorValue* fAmbient; 
    TD3DColorValue* fDiffuse; 
    HRESULT fErrorValue; 
    dword fSize; 
    TD3DColorValue* fSpecular; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DLIGHT7 fD3DLIGHT7; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
