#ifndef TD3DLightingCapsH
#define TD3DLightingCapsH

// ==========================================================================
// File:         TD3DLightingCaps.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DLightingCaps Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"

// ==========================================================================
// Set:         TD3DLightingCapsCaps_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TD3DLightingCaps::Caps property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Caps property description below. 
// ==========================================================================
enum TD3DLightingCaps_Caps_Flags
    {
    d3dlightcaps_directional,
    d3dlightcaps_point,
    d3dlightcaps_spot
    };
typedef Set< TD3DLightingCaps_Caps_Flags, 
             d3dlightcaps_directional, 
             d3dlightcaps_spot >  TD3DLightingCaps_Caps_Set;
// ==========================================================================
// Set:         TD3DLightingCapsLightingModel_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TD3DLightingCaps::LightingModel property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              LightingModel property description below. 
// ==========================================================================
enum TD3DLightingCaps_LightingModel_Flags
    {
    d3dlightingmodel_mono,
    d3dlightingmodel_rgb
    };
typedef Set< TD3DLightingCaps_LightingModel_Flags, 
             d3dlightingmodel_mono, 
             d3dlightingmodel_rgb >  TD3DLightingCaps_LightingModel_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DLightingCaps
// Description: The TD3DLightingCaps component wraps the DirectX 
//              D3DLIGHTINGCAPS structure, which describes the lighting 
//              capabilities of a device. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DLightingCaps : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DLightingCaps : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps_
    // Description: The Caps property defines the types of light source 
    //              supported by the lighting module. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Caps' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Caps 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TD3DLightingCaps_Caps_Set Caps_ = { read=FGetCaps_, 
    write=FSetCaps_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    LightingModel_
    // Description: The LightingModel property defines the lighting model 
    //              being used. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'LightingModel' property via the BCB Object 
    //              Inspector.  For detailed descriptions of each flag, see 
    //              the LightingModel property description below. 
    // ----------------------------------------------------------------------
    __property TD3DLightingCaps_LightingModel_Set LightingModel_ = { 
    read=FGetLightingModel_, write=FSetLightingModel_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    NumLights
    // Description: The NumLights property defines how many lights can be 
    //              supported. 
    // ----------------------------------------------------------------------
    __property dword NumLights = { read=FGetNumLights, write=FSetNumLights, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DLIGHTINGCAPS structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(D3DLIGHTINGCAPS) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DLightingCaps component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps
    // Description: The Caps property defines the types of light source 
    //              supported by the lighting module. 
    //              The described effect applies when the flag is set. 
    // Flags:       D3DLIGHTCAPS_DIRECTIONAL -
    //                Support for directional lights is available. 
    //              D3DLIGHTCAPS_POINT -
    //                Support for point lights is available. 
    //              D3DLIGHTCAPS_SPOT -
    //                Support for spotlights is available. 
    // ----------------------------------------------------------------------
    __property dword Caps = { read=FGetCaps, write=FSetCaps, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DLightingCaps method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    LightingModel
    // Description: The LightingModel property defines the lighting model 
    //              being used. 
    //              The described effect applies when the flag is set. 
    // Flags:       D3DLIGHTINGMODEL_MONO -
    //                The lighting model being utilised is the monochromatic 
    //                or ramp lighting model. 
    //                
    //                The blue color component is converted into grey scale 
    //                and the other values are ignored in the monochrome 
    //                model. 
    //              D3DLIGHTINGMODEL_RGB -
    //                The lighting model being utilised is the RGB or color 
    //                lighting model. 
    // ----------------------------------------------------------------------
    __property dword LightingModel = { read=FGetLightingModel, 
    write=FSetLightingModel, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DLightingCaps's internal D3DLIGHTINGCAPS 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Caps_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Caps property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Caps_Strings = { read=FGetCaps_Strings };

    // ----------------------------------------------------------------------
    // Property:    LightingModel_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the LightingModel property. Note: Any 
    //              memory associated with these strings is released on 
    //              destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* LightingModel_Strings = { read=FGetLightingModel_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DLightingCaps(TComponent* Owner); 
    virtual __fastcall ~TD3DLightingCaps(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DLIGHTINGCAPS* Internal_D3DLIGHTINGCAPS_Ptr = { 
    read=FGetInternal_D3DLIGHTINGCAPS_Ptr, nodefault }; 

    void __fastcall Internal_D3DLIGHTINGCAPS_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TD3DLightingCaps_Caps_Set __fastcall FGetCaps_(); 
    void __fastcall FSetCaps_( TD3DLightingCaps_Caps_Set pCaps_ ); 
    TD3DLightingCaps_LightingModel_Set __fastcall FGetLightingModel_(); 
    void __fastcall FSetLightingModel_( TD3DLightingCaps_LightingModel_Set 
    pLightingModel_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetCaps(); 
    void __fastcall FSetCaps( dword pCaps ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetLightingModel(); 
    void __fastcall FSetLightingModel( dword pLightingModel ); 
    dword __fastcall FGetNumLights(); 
    void __fastcall FSetNumLights( dword pNumLights ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetCaps_Strings();

    TStringList* __fastcall FGetLightingModel_Strings();
// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DLIGHTINGCAPS* __fastcall FGetInternal_D3DLIGHTINGCAPS_Ptr(); 
    void __fastcall FSetInternal_D3DLIGHTINGCAPS( D3DLIGHTINGCAPS* 
    pD3DLIGHTINGCAPS ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fCaps_Strings;
    TStringList* fLightingModel_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DLIGHTINGCAPS fD3DLIGHTINGCAPS; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
