#ifndef TD3DRectH
#define TD3DRectH

// ==========================================================================
// File:         TD3DRect.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               Brian Austin (documentation)
// Description:  This file defines the TD3DRect Component 
//
// "TDx_3DI_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DRect
// Description: The TD3DRect component wraps multiple DirectX D3DRECT 
//              structures, each defining an array of rectangles. 
//              
//              This component is used by the TDx_3DDevice::Clear() method to 
//              define the area to be cleared. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DRect : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DRect : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DRect component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              D3DRECT structures wrapped inside the TD3DRect component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TD3DRect component wraps a single D3DRECT 
    //              structure. To wrap multiple structures, set the 
    //              TD3DRect::ArraySize to the total number of internal 
    //              structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DRect method or fget/fset. 
    //              eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal D3DRECT structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal D3DRECT structures. This result is the same as 
    //              multiplying TD3DRect::ArraySize and TD3DRect::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    X1
    // Description: The X1 property contains the X coordinate of the upper 
    //              left corner of the rectangle. 
    //              
    //              This property is unioned with the x1 property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property long X1[dword pArrayIndex] = { read=FGetX1, write=FSetX1 }; 

    // ----------------------------------------------------------------------
    // Property:    X2
    // Description: The X2 property contains the X coordinate of the lower 
    //              right corner of the rectangle. 
    //              
    //              This property is unioned with the x2 property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property long X2[dword pArrayIndex] = { read=FGetX2, write=FSetX2 }; 

    // ----------------------------------------------------------------------
    // Property:    Y1
    // Description: The Y1 property contains the Y coordinate of the upper 
    //              left corner of the rectangle. 
    //              
    //              This property is unioned with the y1 property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property long Y1[dword pArrayIndex] = { read=FGetY1, write=FSetY1 }; 

    // ----------------------------------------------------------------------
    // Property:    Y2
    // Description: The Y2 property contains the Y coordinate of the lower 
    //              right corner of the rectangle. 
    //              
    //              This property is unioned with the y2 property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property long Y2[dword pArrayIndex] = { read=FGetY2, write=FSetY2 }; 

    // ----------------------------------------------------------------------
    // Property:    x1
    // Description: The x1 property contains the X coordinate of the upper 
    //              left corner of the rectangle. 
    //              
    //              This property is unioned with the X1 property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property long x1[dword pArrayIndex] = { read=FGetx1, write=FSetx1 }; 

    // ----------------------------------------------------------------------
    // Property:    x2
    // Description: The x2 property contains the X coordinate of the lower 
    //              right corner of the rectangle. 
    //              
    //              This property is unioned with the X2 property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property long x2[dword pArrayIndex] = { read=FGetx2, write=FSetx2 }; 

    // ----------------------------------------------------------------------
    // Property:    y1
    // Description: The y1 property contains the Y coordinate of the upper 
    //              left corner of the rectangle. 
    //              
    //              This property is unioned with the Y1 property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property long y1[dword pArrayIndex] = { read=FGety1, write=FSety1 }; 

    // ----------------------------------------------------------------------
    // Property:    y2
    // Description: The y2 property contains the Y coordinate of the lower 
    //              right corner of the rectangle. 
    //              
    //              This property is unioned with the Y2 property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property long y2[dword pArrayIndex] = { read=FGety2, write=FSety2 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DRect's internal D3DRECT structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TD3DRect::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal D3DRECT structures of the TD3DRect 
    //              component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DRect(TComponent* Owner); 
    virtual __fastcall ~TD3DRect(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DRECT* Internal_D3DRECT_Ptr = { 
    read=FGetInternal_D3DRECT_Ptr, nodefault }; 

    void __fastcall Internal_D3DRECT_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    long __fastcall FGetX1( dword pArrayIndex ); 
    void __fastcall FSetX1( dword pArrayIndex, long pX1 ); 
    long __fastcall FGetX2( dword pArrayIndex ); 
    void __fastcall FSetX2( dword pArrayIndex, long pX2 ); 
    long __fastcall FGetY1( dword pArrayIndex ); 
    void __fastcall FSetY1( dword pArrayIndex, long pY1 ); 
    long __fastcall FGetY2( dword pArrayIndex ); 
    void __fastcall FSetY2( dword pArrayIndex, long pY2 ); 
    long __fastcall FGetx1( dword pArrayIndex ); 
    void __fastcall FSetx1( dword pArrayIndex, long px1 ); 
    long __fastcall FGetx2( dword pArrayIndex ); 
    void __fastcall FSetx2( dword pArrayIndex, long px2 ); 
    long __fastcall FGety1( dword pArrayIndex ); 
    void __fastcall FSety1( dword pArrayIndex, long py1 ); 
    long __fastcall FGety2( dword pArrayIndex ); 
    void __fastcall FSety2( dword pArrayIndex, long py2 ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DRECT* __fastcall FGetInternal_D3DRECT_Ptr(); 
    void __fastcall FSetInternal_D3DRECT( dword pArrayIndex, D3DRECT* 
    pD3DRECT ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DRECT* fD3DRECT; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
