#ifndef TDx_Direct3DVertexBufferH
#define TDx_Direct3DVertexBufferH

// ==========================================================================
// File:         TDx_Direct3DVertexBuffer.H
// Author:       Darren Dwyer, BCB_Code_Generator v1.5
// Description:  This file defines the TDx_Direct3DVertexBuffer Component 
//
// "TDx_3DI_Library v1.50"
// (c) 2001 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_3DI_Library_Defns.H"
#include "TDx_3DI_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDx_Direct3DVertexBuffer methods.
class TD3DVertexBufferDesc;
class TDx_3DDevice;
class TDx_Direct3DVertexBuffer;
class TD3DDrawPrimitiveStridedData;

// ==========================================================================
// Class:       TDx_Direct3DVertexBuffer
// Description: The TD3DVertexBufferDesc 
//              
//              The D3DVERTEXBUFFERDESC structure describes the properties of 
//              a vertex buffer object. This structure is used with the 
//              IDirect3D7::CreateVertexBuffer and 
//              IDirect3DVertexBuffer7::GetVertexBufferDesc methods... 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_Direct3DVertexBuffer : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_Direct3DVertexBuffer : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_Direct3DVertexBuffer::Create() method after it has 
    //              successfully created the internal LPDIRECT3DVERTEXBUFFER7 
    //              used within the TDx_Direct3DVertexBuffer component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECT3DVERTEXBUFFER7 
    //              interface. 
    //              
    //              For example, you could retrieve the 
    //              TDx_Direct3DVertexBuffer component's capabilities, 
    //              Create() dependant components, allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_Direct3DVertexBuffer::OnDestroy() event, you can 
    //              fully automate and insulate your code on a per-component 
    //              basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_Direct3DVertexBuffer::Destroy() method, just before 
    //              the TDx_Direct3DVertexBuffer internal 
    //              LPDIRECT3DVERTEXBUFFER7 interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the 
    //              TDx_Direct3DVertexBuffer::OnCreate() event. For example, 
    //              assume you have already setup the 
    //              TDx_Direct3DVertexBuffer::OnCreate() event to retrieve 
    //              the component's capabilities, Create() a dependant 
    //              component and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_Direct3DVertexBuffer::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_Direct3DVertexBuffer component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnMessage()
    // Description: This event is called whenever a message is available. 
    // ----------------------------------------------------------------------
    __property TDx_Message OnMessage = {read=FOnMessage, write=FOnMessage, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnWarning()
    // Description: This event is called when something that may cause an 
    //              error occurs. 
    // ----------------------------------------------------------------------
    __property TDx_Warning OnWarning = {read=FOnWarning, write=FOnWarning, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECT3DVERTEXBUFFER7 used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECT3DVERTEXBUFFER7, call the 
    //              TDx_Direct3DVertexBuffer::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_Direct3DVertexBuffer method 
    //              or fget/fset. eg. D3D_OK or DDERR_SURFACELOST or 
    //              TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECT3DVERTEXBUFFER7 interface used in the 
    //              TDx_Direct3DVertexBuffer component and must be called 
    //              before any methods of the TDx_Direct3DVertexBuffer 
    //              component will function. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create(); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECT3DVERTEXBUFFER7 interface used in the 
    //              TDx_Direct3DVertexBuffer component and should be called 
    //              when the internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetVertexBufferDesc()
    // Description: The IDirect3DVertexBuffer7::GetVertexBufferDesc method 
    //              retrieves a description of the vertex buffer. 
    //              
    //              Return Values 
    //              If the method succeeds, the return value is D3D_OK. 
    //              
    //              If the method fails, the return value can be 
    //              DDERR_INVALIDPARAMS or another error value. 
    // Params:      pVBDesc - 
    //                The VBDesc parameter ...Address of a D3DVERTEXBUFFERDESC 
    //                structure to be filled with a description of the vertex 
    //                buffer. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetVertexBufferDesc( TD3DVertexBufferDesc* 
    pVBDesc ); 

    // ----------------------------------------------------------------------
    // Method:      Lock()
    // Description: The IDirect3DVertexBuffer::Lock methods locks a vertex 
    //              buffer and obtains a pointer to the vertex buffer memory. 
    //              
    //              Return Values 
    //              If the method succeeds, the return value is D3D_OK. 
    //              
    //              If the method fails, the return value can be one of the 
    //              following values: 
    //              
    //              D3DERR_VERTEXBUFFEROPTIMIZED 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    //              DDERR_SURFACEBUSY 
    //              DDERR_SURFACELOST 
    //              
    //              
    //              Remarks 
    //              After locking the vertex buffer, you can access the 
    //              memory until a corresponding call to 
    //              IDirect3DVertexBuffer7::Unlock. 
    //              
    //              You cannot render from a locked vertex buffer; calls to 
    //              the IDirect3DDevice7::DrawIndexedPrimitiveVB or 
    //              IDirect3DDevice7::DrawPrimitiveVB method using a locked 
    //              buffer fail, returning D3DERR_VERTEXBUFFERLOCKED. 
    //              
    //              This method often causes the system to hold the 
    //              Win16Mutex until you call the 
    //              IDirect3DVertexBuffer7::Unlock method. GUI debuggers 
    //              cannot operate while the Win16Mutex is held. 
    //              ... 
    // Params:      pFlags - 
    //                The Flags parameter ...Flags indicating how the vertex 
    //                buffer memory should be locked. 
    //                Flags:
    //                  DDLOCK_DISCARDCONTENTS -
    //                    The DDLOCK_DISCARDCONTENTS Flag is used to...New 
    //                    for DirectX 7.0. Used only with Direct3D 
    //                    vertex-buffer locks. Indicates that no assumptions 
    //                    are made about the contents of the vertex buffer 
    //                    during this lock. This enables Direct3D or the 
    //                    driver to provide an alternative memory area as the 
    //                    vertex buffer. This is useful when you plan to 
    //                    clear the contents of the vertex buffer and fill in 
    //                    new data. 
    //                  DDLOCK_NOOVERWRITE -
    //                    The DDLOCK_NOOVERWRITE Flag is used to...New for 
    //                    DirectX 7.0. Used only with Direct3D vertex-buffer 
    //                    locks. Indicates that no vertices that were 
    //                    referred to in vertex-buffer DrawPrimitive calls 
    //                    since the start of the frame (or the last lock 
    //                    without this flag) are modified during the lock. 
    //                    This can be useful when you want only to append 
    //                    data to the vertex buffer. 
    //                  DDLOCK_NOSYSLOCK -
    //                    The DDLOCK_NOSYSLOCK Flag is used to...If possible, 
    //                    do not take the Win16Mutex (also known as 
    //                    Win16Lock). 
    //                  DDLOCK_READONLY -
    //                    The DDLOCK_READONLY Flag is used to...Indicates 
    //                    that the memory being locked is only to be read 
    //                    from. 
    //                  DDLOCK_SURFACEMEMORYPTR -
    //                    The DDLOCK_SURFACEMEMORYPTR Flag is used 
    //                    to...DIndicates that a valid memory pointer to the 
    //                    vertex buffer is returned; this is the default. 
    //                  DDLOCK_WAIT -
    //                    The DDLOCK_WAIT Flag is used to...If a lock cannot 
    //                    be obtained immediately, the method retries until a 
    //                    lock is obtained or another error occurs. 
    //                  DDLOCK_WRITEONLY -
    //                    The DDLOCK_WRITEONLY Flag is used to...Indicates 
    //                    that the memory being locked is only to be written 
    //                    to. 
    //              pData - 
    //                The Data parameter ...Address of a variable that receives 
    //                the address of the vertex buffer memory if the call 
    //                succeeds. 
    //              pSize - 
    //                The Size parameter ...Address of a variable that receives 
    //                the size of the vertex buffer memory at lplpData. Set to 
    //                NULL if the buffer size is not needed. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Lock( dword pFlags, void** pData, dword* pSize ); 

    // ----------------------------------------------------------------------
    // Method:      Optimize()
    // Description: The Optimize method is used to ...The 
    //              IDirect3DVertexBuffer7::Optimize method converts an 
    //              unoptimized vertex buffer into an optimized vertex 
    //              buffer. 
    //              
    //              Return Values 
    //              If the method succeeds, the return value is D3D_OK. 
    //              
    //              If the method fails, the return value can be one of the 
    //              following values: 
    //              
    //              D3DERR_VERTEXBUFFEROPTIMIZED 
    //              D3DERR_VERTEXBUFFERLOCKED 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    //              
    //              
    //              Remarks 
    //              In the legacy IDirect3DVertexBuffer interface, this 
    //              method accepted a pointer to the IDirect3DDevice3 
    //              interface. 
    // Params:      pD3DDevice - 
    //                The D3DDevice parameter ...Address of the 
    //                IDirect3DDevice7 interface of the device for which this 
    //                vertex buffer is to be optimized. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Optimize( TDx_3DDevice* pD3DDevice ); 

    // ----------------------------------------------------------------------
    // Method:      ProcessVerticies()
    // Description: The IDirect3DVertexBuffer::ProcessVertices method 
    //              processes untransformed vertices into a transformed or 
    //              optimized vertex buffer. 
    //              
    //              Return Values 
    //              If the method succeeds, the return value is D3D_OK. 
    //              
    //              If the method fails, the return value can be one of the 
    //              following values: 
    //              
    //              D3DERR_INVALIDVERTEXFORMAT 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    //              DDERR_SURFACEBUSY 
    //              DDERR_SURFACELOST 
    //              
    //              
    //              Remarks 
    //              Always include the D3DVOP_TRANSFORMED flag in the 
    //              dwVertexOp parameter. If you do not, the method fails, 
    //              returning DDERR_INVALIDPARAMS. 
    //              
    //              In the legacy IDirect3DVertexBuffer interface, this 
    //              method accepted pointers to the IDirect3DDevice3 and 
    //              IDirect3DVertexBuffer interfaces. 
    // Params:      pVertexOp - 
    //                The VertexOp parameter ...Flags defining how the method 
    //                processes the vertices as they are transferred from the 
    //                source buffer. You can specify any combination of the 
    //                following flags: 
    //                Flags:
    //                  D3DVOP_CLIP -
    //                    The D3DVOP_CLIP Flag is used to...Transform the 
    //                    vertices, and clip any vertices that exist outside 
    //                    the viewing frustum. This flag cannot be used with 
    //                    vertex buffers that do not contain clipping 
    //                    information (for example, those created with the 
    //                    D3DVBCAPS_DONOTCLIP flag). 
    //                  D3DVOP_EXTENTS -
    //                    The D3DVOP_EXTENTS Flag is used to...Transform the 
    //                    vertices, then update the extents of the screen 
    //                    rectangle when the vertices are rendered. Using 
    //                    this flag can potentially help performance, but the 
    //                    extents returned by IDirect3DDevice7::GetClipStatus 
    //                    are not updated to account for the vertices when 
    //                    they are rendered. 
    //                  D3DVOP_LIGHT -
    //                    The D3DVOP_LIGHT Flag is used to...Light the 
    //                    vertices. 
    //                  D3DVOP_TRANSFORM -
    //                    The D3DVOP_TRANSFORM Flag is used to...Transform 
    //                    the vertices, using the world, view, and projection 
    //                    matrices. This flag must always be set. 
    //              pDestIndex - 
    //                The DestIndex parameter ...Index into the destination 
    //                vertex buffer (this buffer) where the vertices are placed 
    //                after processing. 
    //              pCount - 
    //                The Count parameter ...Number of vertices in the source 
    //                buffer to process. 
    //              pSrcBuffer - 
    //                The SrcBuffer parameter ...Address of the 
    //                IDirect3DVertexBuffer7 interface for the source vertex 
    //                buffer. 
    //              pSrcIndex - 
    //                The SrcIndex parameter ...Index of the first vertex in 
    //                the source buffer to be processed. 
    //              pD3DDevice - 
    //                The D3DDevice parameter ...Address of the 
    //                IDirect3DDevice7 interface for the device to be used to 
    //                transform the vertices. 
    //              pFlags - 
    //                The Flags parameter ...Processing options. Set this 
    //                parameter to 0 for default processing 
    //                Flags:
    //                  D3DPV_DONOTCOPYDATA -
    //                    The D3DPV_DONOTCOPYDATA Flag is used to...to 
    //                    prevent the system from copying vertex data not 
    //                    affected by the current vertex operation into the 
    //                    destination buffer. For more information, see 
    //                    Processing Vertices. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall ProcessVerticies( dword pVertexOp, dword 
    pDestIndex, dword pCount, TDx_Direct3DVertexBuffer* pSrcBuffer, dword 
    pSrcIndex, TDx_3DDevice* pD3DDevice, dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      ProcessVerticiesStrided()
    // Description: The IDirect3DVertexBuffer::ProcessVerticesStrided method 
    //              processes untransformed strided vertices into a 
    //              transformed or optimized vertex buffer. The 
    //              ProcessVerticiesStrided method is used to ... 
    //              
    //              Return Values 
    //              If the method succeeds, the return value is D3D_OK. 
    //              
    //              If the method fails, the return value can be one of the 
    //              following values: 
    //              
    //              D3DERR_INVALIDVERTEXFORMAT 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    //              DDERR_SURFACEBUSY 
    //              DDERR_SURFACELOST 
    //              
    //              
    //              Remarks 
    //              Always include the D3DVOP_TRANSFORMED flag in the 
    //              dwVertexOp parameter. If you do not, the method fails, 
    //              returning DDERR_INVALIDPARAMS. 
    //              
    //              This method was introduced with the 
    //              IDirect3DVertexBuffer7 interface. 
    // Params:      pVertexOp - 
    //                The VertexOp parameter ...Flags defining how the method 
    //                processes the vertices as they are transferred from the 
    //                source buffer. You can specify any combination of the 
    //                following flags: 
    //                Flags:
    //                  D3DVOP_CLIP -
    //                    The D3DVOP_CLIP Flag is used to...Transform the 
    //                    vertices, and clip any vertices that exist outside 
    //                    the viewing frustum. This flag cannot be used with 
    //                    vertex buffers that do not contain clipping 
    //                    information (for example, those created with the 
    //                    D3DVBCAPS_DONOTCLIP flag). 
    //                  D3DVOP_EXTENTS -
    //                    The D3DVOP_EXTENTS Flag is used to...Transform the 
    //                    vertices, then update the extents of the screen 
    //                    rectangle when the vertices are rendered. Using 
    //                    this flag can potentially help performance, but the 
    //                    extents returned by IDirect3DDevice7::GetClipStatus 
    //                    are not updated to account for the vertices when 
    //                    they are rendered. 
    //                  D3DVOP_LIGHT -
    //                    The D3DVOP_LIGHT Flag is used to...Light the 
    //                    vertices. 
    //                  D3DVOP_TRANSFORM -
    //                    The D3DVOP_TRANSFORM Flag is used to...Transform 
    //                    the vertices, using the world, view, and projection 
    //                    matrices. This flag must always be set. 
    //              pDestIndex - 
    //                The DestIndex parameter ...Index into the destination 
    //                vertex buffer (this buffer) where the vertices are placed 
    //                after processing. 
    //              pCount - 
    //                The Count parameter ...Number of vertices in the source 
    //                buffer to process. 
    //              pVertexArray - 
    //                The VertexArray parameter ...Array of 
    //                D3DDRAWPRIMITIVESTRIDEDDATA structures that contains the 
    //                vertices for this primitive. 
    //              pSrcIndex - 
    //                The SrcIndex parameter ..Index of the first vertex in the 
    //                source buffer to be processed. . 
    //              pD3DDevice - 
    //                The D3DDevice parameter ...Address of the 
    //                IDirect3DDevice7 interface for the device to be used to 
    //                transform the vertices. 
    //              pFlags - 
    //                The Flags parameter ...Processing options. Set this 
    //                parameter to 0 for default processing. 
    //                Flags:
    //                  DONOTCOPYDATA -
    //                    DONOTCOPYDATA ... to prevent the system from 
    //                    copying vertex data not affected by the current 
    //                    vertex operation into the destination buffer. For 
    //                    more information, see Processing Vertices. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall ProcessVerticiesStrided( dword pVertexOp, dword 
    pDestIndex, dword pCount, TD3DDrawPrimitiveStridedData* pVertexArray, 
    dword pSrcIndex, TDx_3DDevice* pD3DDevice, dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      Unlock()
    // Description: The IDirect3DVertexBuffer7::Unlock method unlocks a 
    //              previously locked vertex buffer.   d is used to ... 
    //              
    //              Return Values 
    //              If the method succeeds, the return value is D3D_OK. 
    //              
    //              If the method fails, the return value can be one of the 
    //              following values: 
    //              
    //              D3DERR_VERTEXBUFFERUNLOCKFAILED   
    //              DDERR_GENERIC 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_SURFACEBUSY 
    //              DDERR_SURFACELOST 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Unlock(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_Direct3DVertexBuffer(TComponent* Owner); 
    virtual __fastcall ~TDx_Direct3DVertexBuffer(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_3DI_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECT3DVERTEXBUFFER7 Internal_LPDIRECT3DVERTEXBUFFER7 = { 
    read=FGetInternal_LPDIRECT3DVERTEXBUFFER7, 
    write=FSetInternal_LPDIRECT3DVERTEXBUFFER7, nodefault }; 
    __property LPDIRECT3DVERTEXBUFFER7* Internal_LPDIRECT3DVERTEXBUFFER7_Ptr 
    = { read=FGetInternal_LPDIRECT3DVERTEXBUFFER7_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECT3DVERTEXBUFFER7_Update(); 
    __property LPDIRECT3DVERTEXBUFFER Internal_LPDIRECT3DVERTEXBUFFER = { 
    read=FGetInternal_LPDIRECT3DVERTEXBUFFER, 
    write=FSetInternal_LPDIRECT3DVERTEXBUFFER, nodefault }; 
    __property LPDIRECT3DVERTEXBUFFER* Internal_LPDIRECT3DVERTEXBUFFER_Ptr = 
    { read=FGetInternal_LPDIRECT3DVERTEXBUFFER_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECT3DVERTEXBUFFER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECT3DVERTEXBUFFER7 __fastcall 
    FGetInternal_LPDIRECT3DVERTEXBUFFER7(); 
    void __fastcall FSetInternal_LPDIRECT3DVERTEXBUFFER7( 
    LPDIRECT3DVERTEXBUFFER7 pLPDIRECT3DVERTEXBUFFER7 ); 
    LPDIRECT3DVERTEXBUFFER7* __fastcall 
    FGetInternal_LPDIRECT3DVERTEXBUFFER7_Ptr(); 
    LPDIRECT3DVERTEXBUFFER __fastcall FGetInternal_LPDIRECT3DVERTEXBUFFER(); 
    void __fastcall FSetInternal_LPDIRECT3DVERTEXBUFFER( 
    LPDIRECT3DVERTEXBUFFER pLPDIRECT3DVERTEXBUFFER ); 
    LPDIRECT3DVERTEXBUFFER* __fastcall 
    FGetInternal_LPDIRECT3DVERTEXBUFFER_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECT3DVERTEXBUFFER7 fLPDIRECT3DVERTEXBUFFER7; 
    LPDIRECT3DVERTEXBUFFER fLPDIRECT3DVERTEXBUFFER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;
    TDx_Message FOnMessage;
    TDx_Warning FOnWarning;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
